%% Generate user data
% by Ion Marques, ion.marques@ehu.es

userlist = {};
name = char;
material = zeros(5,1);
dirtiness = zeros(4,1);
color = zeros(3,1);
auser = struct('name',name,'material',material,'dirtiness',dirtiness,'color',color);

% Examples. Add more if you want. Numeric data are percentages.
namelist = char('Butcher','Athlete','Hotel','Executive','Sheepherder','Clown');
% Matarials: Synthetic, Silk, Bedding, Cotton, Wool
materiallist = [
    10 0 0 90 0
    90 0 0 10 0
    0 0 100 0 0
    0 80 0 20 0
    10 0 0 0 90
    50 0 0 50 0
    ];
% Dirtiness: Less, Normal, High, Very stained
dirtinesslist = [
    0 0 20 80
    80 20 0 0
    90 10 0 0
    100 0 0 0
    0 80 20 0
    60 0 20 20
    ];
% Color: White, little colors, very colored
colorlist = [
    90 0 10
    0 50 50
    100 0 0
    50 50 0
    90 10 0
    0 10 90
    ];

% Check for errors in the data
numusers = size(namelist,1);
if size(materiallist)==[numusers,5] & ...
        size(dirtinesslist)==[numusers,4] & ...
        size(colorlist)==[numusers,3]
else
    error('Example data is wrong. Some value may be missing. Check the matrixes.');disp('lolo');
end
if sum(materiallist,2) == repmat(100,numusers,1) &  ...
        sum(dirtinesslist,2) == repmat(100,numusers,1) &  ...
        sum(colorlist,2) == repmat(100,numusers,1)
else
    error('Example data is wrong. Some row(s) dont add up to 100.');
end

userlist = cell(numusers,1);
for i=1:numusers
    auser.name = namelist(i,:);
    auser.material = materiallist(i,:);
    auser.dirtiness = dirtinesslist(i,:);
    auser.color = colorlist(i,:);
    userlist{i} = auser;
end


%% Generate triplet task-recipe-satisfaction data

% Data type.

atask = struct('material',material,'dirtiness',dirtiness,'color',color);
arecipe = zeros(1,5); % these five values are: L,?C,RPM,ML,min
asatisfaction = 0; % a value between 1 and 5
atriplet = struct('task',atask,'recipe',arecipe,'satisfaction',asatisfaction);
omega = {}; % this will be the list of triplets, ie our data base


% Create ideal recipes, ie seed recipes, for our six users

seedprogram(1,:) = [10 90 1500 30 180];
seedprogram(2,:) = [17 50 1200 15 090];
seedprogram(3,:) = [10 60 1600 10 180];
seedprogram(4,:) = [20 40 0400 10 060];
seedprogram(5,:) = [18 40 0700 20 070];
seedprogram(6,:) = [09 30 1250 20 140];

% Main data creation loop
nT = 99; % number of non-ideal tasks per subject to be generated
satisIndex = zeros(numusers+nT*numusers,1);
satisList = zeros(numusers+nT*numusers,1);
omega = cell(1,numusers+nT*numusers);
k = 0;

for i=1:numusers
    taskaux.material = userlist{i}.material;
    taskaux.dirtiness = userlist{i}.dirtiness;
    taskaux.color = userlist{i}.color;
    recipeaux = seedprogram(i,:);
    tripletaux.task = taskaux;
    tripletaux.recipe = recipeaux;
    tripletaux.satisfaction = 5;
    k = k + 1;
   
    satisList(k) = tripletaux.satisfaction;

    omega{k} = tripletaux;
    % Random task variation locations (what changes?)
    rM = sprand(99,5,0.3);
    rD = sprand(99,4,0.3);
    rC = sprand(99,3,0.3);
    % Random recipe variation location and rates
    rP =  0.5+(1.5-0.5).*sprand(99,5,0.3);
    rP = full(rP);
    rP(rP==0.5) = 1;
    for j=1:nT
        % crate noised task
        material = userlist{i}.material;    
        material(rM(j,:)~=0) = material(rM(j,:)~=0)+10;
        [maxm,imax] = max(material);
        material(imax) = maxm-10*sum(rM(j,:)~=0);
        atask.material = material;
        dirtiness = userlist{i}.dirtiness;    
        dirtiness(rD(j,:)~=0) = dirtiness(rD(j,:)~=0)+10;
        [maxm,imax] = max(dirtiness);
        dirtiness(imax) = maxm-10*sum(rD(j,:)~=0);
        atask.dirtiness = dirtiness;
        color = userlist{i}.color;    
        color(rC(j,:)~=0) = color(rC(j,:)~=0)+10;
        [maxm,imax] = max(color);
        color(imax) = maxm-10*sum(rC(j,:)~=0);
        atask.color = color;
        % create noised recipe      
        arecipe = seedprogram(i,:);
        rPaux = rP(j,:);
        arecipe = arecipe.*rPaux;
        % create noised feedback
        satisMatTemp = 5;
        if (arecipe(2)>30)&&((material(2)>0)||(material(5)>0)) % temp-material
            satisA = 5 - material(2)*(0.03+(arecipe(2)-30)*0.002);
            satisB = 5 - material(5)*(0.03+(arecipe(2)-30)*0.002);
            satisMatTemp = min(satisA,satisB);
        end
        satisColTemp = 5;
        if ((arecipe(2)>60)&&(color(2)>0))||((arecipe(2)>30)&&(color(3)>0)) %temp-color
            satisA = 5 - color(2)*(0.01+(arecipe(2)-60)*0.002);
            satisB = 5 - color(3)*(0.01+(arecipe(2)-30)*0.002);
            satisColTemp = min(satisA,satisB);
        end
        satisRPM = 5;
        if ((arecipe(3)>700)&&(material(2)>0))||((arecipe(3)>500)&&(material(5)>0))  % rpm too high
            satisA = 5 - material(2)*(0.001+(arecipe(3)-700)*0.0002);
            satisB = 5 - material(5)*(0.001+(arecipe(3)-500)*0.0002);
            satisRPM = min(satisA,satisB);
        end
        satisfaction = min([satisMatTemp satisColTemp satisRPM]);
        if satisfaction == 5 
            
            satisIndex(k+1) = 1;
            
            distTask = sum([abs(atask.material-userlist{i}.material) ...
                abs(atask.dirtiness-userlist{i}.dirtiness) ...
                abs(atask.color-userlist{i}.color)]);            
            distRec = sum((arecipe-seedprogram(i,:)).*[30 20 1 20 10]); % scaling
            distRatio = (distTask*10-distRec)*0.001;
            satisfaction = 5 - distRatio;
        end
        tripletaux.task = atask;
        tripletaux.recipe = arecipe;        
        satisfaction = min(satisfaction,5);
        satisfaction = max(satisfaction,0);
        tripletaux.satisfaction = satisfaction;       
        k = k + 1;   
        
        satisList(k) = satisfaction;
        
        omega{k} = tripletaux;
    end
end  
figure
subplot(3,1,1);
hist(satisList(satisIndex==1));
subplot(3,1,2);
hist(satisList(satisIndex==0));
subplot(3,1,3);
hist(satisList,25);
iptsetpref('ImshowBorder','tight');